import datetime
import html
import re

from bs4 import BeautifulSoup

from email_scrapper.models import Order, Item, Stores

global_remover = re.compile("(=(?<==)(.*?)(?=\\s))", flags=re.DOTALL)


def parse_lego_email(email) -> Order:
    soup = BeautifulSoup(str(email), "lxml")
    if re.search("Order Confirmation", soup.text):
        email_date = email.get("date")
        try:

            order_date = datetime.datetime.strptime(email_date, "%d %b %Y %H:%M:%S %z")
        except Exception as e:
            order_date = datetime.datetime.strptime(email_date, "%a, %d %b %Y %H:%M:%S %z")
        order_number = None
        items = []
        cart = []
        prices = []
        quantites = []
        order_discount = 0.00
        discounts = set(re.findall("-CDN\$\s.*", str(email)))
        for discount in discounts:
            try:
                amount = float(discount[6:])
            except:
                amount = 0
            order_discount += amount
        all_td_tags = soup.find_all("td")
        for index, data in enumerate(all_td_tags):
            text = re.sub("\t", "", html.unescape(data.text))
            if index in [1, 2, 3, 4]:
                continue
            if "Order Number" in text:
                first_search = re.search("(T.*)", text)
                order_number = first_search.group(0)
                break

        item_names = soup.find_all("td", attrs={"class": "3D=22padT15=22"})
        for item in item_names:
            name = re.sub(global_remover, "", item.text)
            name = re.sub("\n", "", name)
            name = " ".join([r.group(0) for r in re.finditer("(\d*?\w+)", name)])
            items.append(name)
            continue
        item_prices = soup.find_all("td", attrs={"class": "3D=22w50pc"})
        for price in item_prices:
            text = price.text
            if re.search("Qty", text):
                quantites.append(re.search(r"\d+", text).group(0))
                continue
            _ = re.sub("=2E", ".", text)
            _ = re.sub(global_remover, "", _)
            item_price = re.search("(\d+.\d+)", _)
            prices.append(float(item_price.group(0)))

        for item, quantity, price in zip(items, quantites, prices):
            try:
                unit_price = float(price) / float(quantity)
            except ZeroDivisionError:
                unit_price = float(price)
            cart.append(
                Item(item, unit_price, int(quantity), order_number))

        return Order(order_number, order_date, Stores.LEGOCA, cart, discount=order_discount)

    return None



import datetime
import html
import io
import re

import pdfminer
import pdfminer.high_level
import pdfminer.layout
import pdfminer.settings
from bs4 import BeautifulSoup
from pdfminer.image import ImageWriter

from email_scrapper.models import Order, Item, Stores

pdfminer.settings.STRICT = False


class BestBuyReader:

    def save_attachment(self, msg):
        """
        Given a message, save its attachments to the specified
        download folder (default is /tmp)

        return: file path to attachment
        """
        files = {}
        for part in msg.walk():
            if part.get_content_maintype() == 'multipart':
                continue
            if part.get('Content-Disposition') is None:
                continue
            filename = part.get_filename()
            try:
                fp = io.BytesIO(part.get_payload(decode=True))
                fp.seek(0)
                files[filename] = fp
            except Exception as e:
                print(e)
                continue
        if len(files) == 0:
            return {}
        converted_data = self.extract_text(files=files, output_type="html")
        return self.parse_pdf(converted_data)

    def extract_text(self, files=None,
                     _py2_no_more_posargs=None,  # Bloody Python2 needs a shim
                     no_laparams=False, all_texts=None, detect_vertical=None,  # LAParams
                     word_margin=None, char_margin=None, line_margin=None, boxes_flow=None,  # LAParams
                     output_type='text', codec='utf-8', strip_control=False,
                     maxpages=0, page_numbers=None, password="", scale=1.0, rotation=0,
                     layoutmode='normal', output_dir=None, debug=False,
                     disable_caching=False, **other):
        if _py2_no_more_posargs is not None:
            raise ValueError("Too many positional arguments passed.")
        if not files:
            raise ValueError("Must provide files to work upon!")

        # If any LAParams group arguments were passed, create an LAParams object and
        # populate with given args. Otherwise, set it to None.
        if not no_laparams:
            laparams = pdfminer.layout.LAParams()
            for param in ("all_texts", "detect_vertical", "word_margin", "char_margin", "line_margin", "boxes_flow"):
                paramv = locals().get(param, None)
                if paramv is not None:
                    setattr(laparams, param, paramv)
        else:
            laparams = None

        imagewriter = None
        if output_dir:
            imagewriter = ImageWriter(output_dir)

        out_files = []

        for fname, fdata in files.items():
            file = io.BytesIO()
            pdfminer.high_level.extract_text_to_fp(fdata, file, **locals())
            file.seek(0)
            out_files.append(file)
        return out_files

    def bs4method(self, raw_data):
        soup = BeautifulSoup(raw_data, "lxml")
        all_spans = soup.find_all("span")
        quantities = []
        items = []
        prices = []
        order_number = None
        cart = []
        item_name = None
        order_date = None
        order_discount = 0.00
        discounts = set(re.findall("-CDN\$\s.*", raw_data))
        for discount in discounts:
            try:
                amount = float(discount[6:])
            except:
                amount = 0
            order_discount += amount

        for index, data in enumerate(all_spans):
            if index == 0:
                continue
            else:
                if "Order Number" in all_spans[index - 1].text:
                    order_number = re.search(r'(\d*)', data.text).group(0)
                elif "Qty" in all_spans[index - 1].text:
                    for qty in re.findall(r"\d+", data.text):
                        quantities.append(int(qty))
                elif "Product Description" in data.text or "Product Description" in all_spans[index - 1].text:
                    if item_name is not None:
                        continue
                    try:
                        tmp_items = re.findall(r"(?s)\n(.*?)\n", data.text)
                        text = data.text.split("\n")
                        if not tmp_items:
                            raise Exception
                        if tmp_items:
                            item_name = tmp_items[0]
                        if "Product Description" in text:
                            item_name = tmp_items[0]
                            raise Exception
                        items.extend(t for t in data.text.split("\n") if t)
                    except:
                        if data.text.strip("\n") in ["Product Description", "Payment Information", "Serial Number"]:
                            continue
                        item_name = html.unescape(
                            item_name.strip("\n") if item_name is not None else data.text.strip("\n"))
                        items.append(item_name)
                elif "Order Date" in data.text:
                    order_date = datetime.datetime.strptime(data.text.strip("\n"),
                                                            "Order Date: %d-%b-%Y %I:%M:%S %p (PST)")
                elif "Total\n" == all_spans[index - 1].text:
                    for total in re.findall(r"\d+.\d+", data.text):
                        prices.append(float(total.replace(",", "")))

        for item, quantity, price in zip(items, quantities, prices):
            try:
                unit_price = float(price) / int(quantity)
            except ZeroDivisionError:
                unit_price = float(price)
            cart.append(Item(item, unit_price, quantity, order_number))

        return Order(order_number, order_date, Stores.BESTBUYCA, cart,discount=order_discount)

    def parse_pdf(self, files: list):
        items = []
        order_date = None
        order_number = None
        for file in files:
            raw_data = file.read().decode("utf-8")
            data = self.bs4method(raw_data)
            if order_date is None:
                order_date = data.purchased
            if order_number is None:
                order_number = data.id
            items.extend(data.cart)

        return Order(order_number, order_date, Stores.BESTBUYCA, items)



import datetime
import logging
import typing

from email_scrapper import utils
from email_scrapper.models import Stores, Order
from email_scrapper.stores import lego, ebgames, walmart, amazon
from email_scrapper.stores.bestbuy import BestBuyReader

logger = logging.getLogger(__name__)


class BaseReader:

    def __init__(self, user_email: str = None, date_from: datetime.datetime = None,
                 email_mapping: typing.Dict[Stores, str] = None):
        self.stores: typing.Dict[Stores, typing.List[Order]] = {}
        self.email = user_email
        if date_from:
            self.search_date_range = date_from
        else:
            self.search_date_range = datetime.datetime.now() - datetime.timedelta(days=7)
        self._store_mapping: typing.Mapping[Stores, typing.Callable] = {
            Stores.AMAZONCA: self.get_amazon_ca,
            Stores.BESTBUYCA: self.get_best_buy,
            Stores.EBGAMES: self.get_ebgames,
            Stores.LEGOCA: self.get_lego,
            Stores.WALMART: self.get_walmart
        }
        self._orders: typing.Dict[str, Order] = {}
        self._email_mapping = email_mapping or {}

    def _get_search_date_range(self):
        return self.search_date_range.strftime(
            "%d-%b-%Y")

    def _get_store(self, store: Stores) -> typing.Optional[typing.List[Order]]:
        func = self._store_mapping.get(store)
        if func:
            return func()
        else:
            return None

    def read_store_emails(self, store: Stores, subject: str = None) -> typing.Iterable[str]:
        raise NotImplemented

    def _save_order(self, order: Order):
        if order.id in self._orders:
            _order = self._orders.get(order.id)
            _order += order
        else:
            self._orders[order.id] = order

    def _finish(self):
        raise NotImplemented

    def _login(self):
        raise NotImplementedError

    def run(self) -> typing.List[Order]:
        self._login()
        return_data = []
        stores = [store for store in Stores]
        for store in stores:
            store_data = self._get_store_email(store)
            if store_data:
                return_data.extend(store_data)
        self._finish()
        return return_data

    def _get_user_email(self) -> str:
        return self.email

    def get_amazon_ca(self) -> typing.List[Order]:
        return self._email_processor(Stores.AMAZONCA, amazon.get_data)

    def get_best_buy(self) -> typing.List[Order]:
        return self._email_processor(Stores.BESTBUYCA, BestBuyReader().save_attachment, "ship")

    def get_ebgames(self) -> typing.List[Order]:
        return self._email_processor(Stores.EBGAMES, ebgames.parse_ebgames_email)

    def get_lego(self) -> typing.List[Order]:
        return self._email_processor(Stores.LEGOCA, lego.parse_lego_email)

    def get_walmart(self) -> typing.List[Order]:
        return self._email_processor(Stores.WALMART, walmart.parse_walmart_email, "shipped")

    def _get_search_query(self, store: Stores, subject: str = None):
        email_address = self._get_store_email(store)
        if subject:
            subject = f"SUBJECT {subject}"
        search_query_list = [
            f"FROM '{email_address}'",
            subject,
            f"SINCE {self._get_search_date_range()}",
            f"TO {self._get_user_email()}"
        ]
        search_query = f"({' '.join(q for q in search_query_list if q)})"
        return search_query

    def _get_store_email(self, store: Stores) -> str:
        """

        Parameters
        ----------
        store :class:Stores

        Returns
        -------
        the email of the store that the reader will filter by

        """
        if self._email_mapping:
            email = self._email_mapping.get(store)
            if email:
                return email
        else:
            return utils.get_store_email(store).value

    def _email_processor(self, store: Stores, processor: typing.Callable[[str], Order], subject: str = None) -> \
            typing.List[Order]:
        logger.log(logging.INFO, f"Processing {store}")
        self._orders.clear()
        for msg_body in self.read_store_emails(store, subject):
            try:
                new_order = processor(msg_body)
                if len(new_order) > 0:
                    self._save_order(new_order)
            except TypeError:
                continue
            except Exception as e:
                logger.log(logging.ERROR, e)
                continue
        else:
            self.stores[store] = list(self._orders.values())
        return self.stores[store]



import datetime
import html
import logging
import re

from bs4 import BeautifulSoup

from email_scrapper.models import Order, Item, Stores

global_remover = re.compile("(=(?<==)(.*?)(?=\\s))", flags=re.DOTALL)

logger = logging.getLogger(__name__)


def does_item_exist(items, item):
    try:
        item = item.group(0)
    except:
        item = item

    for i in items:
        if item.replace(" ", "") == i.replace(" ", ""):
            return True
    else:
        return False


def get_data(email):
    email = str(email)
    soup = BeautifulSoup(email, "lxml")
    order_number = re.search(r'(?<=Order\s#)(\d*-\d*-\d*)', email).group(1)
    date = datetime.datetime.strptime(re.findall(r'(Date:.*)', email)[1][6:], "%a, %d %b %Y %H:%M:%S %z")
    items = []
    order_discount = 0.00
    item_quanitites = []
    prices = [p.text for p in soup.find_all("strong") if "CDN" in p.text]
    cart = []
    discounts = set(re.findall("-CDN\$\s.*", email))
    for discount in discounts:
        try:
            amount = float(discount[6:])
        except:
            amount = 0
        order_discount += amount

    item_names = [td.text for td in [td.find("a") for td in soup.find_all("td") if re.search(r"(Sold)", td.text)
                                     or re.search(r"(S=\nold)", td.text)] if len(td.text) > 5]

    for index, link in enumerate(item_names):
        tmp = re.sub(global_remover, "", link)
        tmp = re.sub("\n", "", tmp)
        possible_items = re.sub(r"(\s{2,})", " ", tmp)
        possible_items = re.sub(r"CDN\$ \d+\.\d+", "", possible_items)
        possible_items = possible_items.split("  ")
        if len(possible_items) > 1 and re.search(r"(\d+)(?=([Xx])?)", possible_items[0]):
            for it in possible_items:
                try:
                    item_name = re.search(r"(\dx)(?<=\dx)(.*)(?=Sold)", it)
                    if item_name is not None:
                        item_name = item_name.group(2)
                    else:
                        item_name = re.sub(r"\d+x", "", it)
                except Exception as e:
                    print(e)
                if not does_item_exist(items, item_name):
                    items.append(html.unescape(re.sub(r"\s{2,}", "", item_name)))
                else:
                    continue
                item_quantity = re.search(r"(\d+)(?=([Xx])(=)?)", it)
                if item_quantity:
                    try:
                        item_quanitites.append(int(item_quantity.group(0)))
                    except:
                        item_quanitites.append(1)
                else:
                    item_quanitites.append(1)
        else:
            item_name = re.sub("\n", "", tmp)
            item_name = re.sub(r"(=a.*)", "", item_name)
            item_name = html.unescape(item_name)
            if not does_item_exist(items, item_name):
                item_quantity = re.search(r"(\d+)(?=([Xx])(=)?)", tmp)
                if item_quantity:
                    try:
                        item_quanitites.append(int(item_quantity.group(0)))
                    except:
                        item_quanitites.append(1)
                else:
                    item_quantity = re.search(r"(\d+)(?=([Xx])(=)?)", item_name)
                    if item_quantity:
                        item_quanitites.append(int(item_quantity.group(0)))
                    else:
                        item_quanitites.append(1)
                item_name = re.sub(r'(\d+)(?=([Xx])(=)?)([Xx])', "", item_name)
                item_name = re.sub(r"(<.*?>)", "", item_name)
                item_name = re.sub(r"CDN\$ \d+\.\d+", "", item_name)
                items.append(html.unescape(re.sub(r"\s{2,}", "", item_name)))
            else:
                continue
    for item, price, quantity in zip(items, prices, item_quanitites):
        item = re.sub(r"(Sold by Amazon\.com\.ca, [Ii]nc\. )", "", item)
        item = re.sub(r"(Sold by Amazon\.com\.ca, [Ii]nc\.)", "", item)
        item = re.sub(r"(<.*?>)", "", item)
        if quantity == 0:
            quantity = 10
        formated_price = re.search(r'(\d+.\d+)', price) or re.search(r"(\d+.\d+.\d+)", price)
        try:
            total_price = float(formated_price.group(0).replace(",", ""))
            try:
                unit_price = round(total_price / quantity, 2)
            except ZeroDivisionError:
                unit_price = total_price
            cart.append(Item(item, unit_price, quantity, order_number))
        except Exception as e:
            logger.log(logging.ERROR, e)

    rdata = Order(order_number, date, Stores.AMAZONCA, cart, discount=order_discount)
    return rdata



import datetime
import html
import re

from bs4 import BeautifulSoup

from email_scrapper.models import Order, Item, Stores

global_remover = re.compile("(=(?<==)(.*?)(?=\\s))", flags=re.DOTALL)


def parse_ebgames_email(email):
    soup = BeautifulSoup(str(email), "lxml")
    email_date = email.get("date")
    try:

        order_date = datetime.datetime.strptime(email_date, "%d %b %Y %H:%M:%S %z")
    except Exception as e:
        order_date = datetime.datetime.strptime(email_date, "%a, %d %b %Y %H:%M:%S %z")
    table_fields = {0: "Sku", 1: "Item", 2: "Platform", 3: "quantity", 4: "price"}
    order_number = None
    items = []
    quantities = []
    prices = []
    cart = []
    all_td_tags = soup.find_all("td")
    all_p_tags = soup.find_all("p")
    order_discount = 0.00
    discounts = set(re.findall("-CDN\$\s.*", str(email)))
    for discount in discounts:
        try:
            amount = float(discount[6:])
        except:
            amount = 0
        order_discount += amount
    for row in all_p_tags[2:]:
        if "Order number" in row.text:
            first_search = re.search(r"(?s)=0A(.*?)\|", row.text) or re.search(r"(?<=\=0A)(.*?)(?=\=7C)", row.text)
            order_number = re.search(r"(\d{2,})", first_search.group(0))
            if order_number is not None:
                order_number = order_number.group(0)
    for index, row in enumerate(all_td_tags):
        text = html.unescape(row.text)
        if index - 5 == 0:
            continue
        elif index - 5 == 1:
            items.append(re.sub(global_remover, "", text))
        elif index - 5 == 2:
            continue
        elif index - 5 == 3:
            try:
                quantities.append(int(text))
            except:
                quantities.append(1)
        elif index - 5 == 4:
            _ = re.sub("=2E", ".", text)
            _ = re.sub(global_remover, "", _)
            prices.append(float(re.sub("\\n", "", _)))

    for item, quantity, price in zip(items, quantities, prices):
        try:
            unit_price = float(int(price) / int(quantity))
        except ZeroDivisionError:
            unit_price = float(price)
        cart.append(Item(item, unit_price, quantity, order_number))

    if order_number is not None:
        return Order(order_number, order_date, Stores.EBGAMES,cart,discount=order_discount)
    else:
        return None



import datetime
import typing
from difflib import SequenceMatcher
from enum import Enum, auto


class Stores(Enum):
    AMAZONCA = auto()
    AMAZONCOM = auto()
    BESTBUYCA = auto()
    BESTBUYCOM = auto()
    LEGOCA = auto()
    WALMART = auto()
    EBGAMES = auto()


class StoreEmail(Enum):
    AMAZONCA = "shipment-tracking@amazon.ca"
    BESTBUYCA = "noreploy@bestbuy.ca"
    EBGAMES = "help@ebgames.ca"
    LEGOCA = "legoshop@e.lego.com"
    WALMART = "noreply@walmart.ca"


class Order:

    def __init__(self, order_number: str, purchased: datetime.datetime, store: Stores,
                 cart: typing.List["Item"] = None,
                 tracking: str = None, shipped: bool = None, discount: float = 0.00):
        self.id = order_number
        self.purchased = purchased
        self.store = store
        self.tracking = tracking
        self.shipped = shipped
        self.discount = round(discount, 2)
        self.cart = cart or []

    def __repr__(self):
        return f"{self.id} - {self.store}"

    def __len__(self):
        return len(self.cart)

    def __lt__(self, other):
        return self.purchased < other.date

    def __gt__(self, other):
        return self.purchased > other.date

    def __hash__(self):
        return int(self.id.replace("-", ""))

    def __eq__(self, other):
        same_order = self.id = other.id
        same_cart = self.cart == other.cart
        return all([same_order, same_cart])

    def __getitem__(self, item):
        return self.__dict__[item]

    def __iadd__(self, other: "Order"):
        for item1, item2 in zip(self.cart, other.cart):
            ratio = SequenceMatcher(None, item1.name, item2.name).ratio()
            if ratio > 0.90:
                item1.quantity += item2.quantity

    def __iter__(self):
        iters = dict((x, y) for x, y in Order.__dict__.items() if not x.startswith("_"))
        iters.update(self.__dict__)
        for x, y in iters.items():
            if isinstance(y, list):
                yield x, [dict(item) for item in y]
            elif isinstance(y, Stores):
                yield x, y.value
            elif isinstance(y, datetime.datetime):
                yield x, y.strftime("%Y-%m-%d")
            else:
                yield x, y


class Item:
    def __init__(self, name, unit_price: float, quantity: int, order_id: str, item_page: str = None):
        self.name = name
        self.order = order_id
        self.unit_price = round(unit_price, 2)
        self.quantity = quantity
        self.item_page = item_page

    def __repr__(self):
        return f"<{self.name}> - <{self.order}> - <{self.quantity}> - <{self.unit_price}>"

    def __iter__(self):
        iters = dict((x, y) for x, y in Item.__dict__.items() if not x.startswith("_"))
        iters.update(self.__dict__)
        for x, y in iters.items():
            if isinstance(y, datetime.datetime):
                yield x, y.strftime("%Y-%m-%d")
            else:
                yield x, y

    def __eq__(self, other):
        return self.order == other.order and self.name == other.name

    def __hash__(self):
        return hash(self.name)



import typing

from email_scrapper.models import Stores, StoreEmail, Order

store_email_mapping = {
    Stores.AMAZONCA: StoreEmail.AMAZONCA,
    Stores.BESTBUYCA: StoreEmail.BESTBUYCA,
    Stores.EBGAMES: StoreEmail.EBGAMES,
    Stores.LEGOCA: StoreEmail.LEGOCA,
    Stores.WALMART: StoreEmail.WALMART
}


def get_store_email(store: Stores) -> StoreEmail:
    return store_email_mapping[store]


def store_to_dict(store_data: typing.List["Order"]) -> list:
    if store_data:
        return [dict(order) for order in store_data]
    return []



import datetime
import email
import imaplib
import logging
import typing

from email_scrapper.email_settings import Email
from email_scrapper.models import Stores
from email_scrapper.readers.base_reader import BaseReader

logger = logging.getLogger(__name__)


class SmtpReader(BaseReader):

    def __init__(self, username: str, password: str, settings: Email = Email.GMAIL, email_address: str = None,
                 locations: typing.Dict[Stores, str] = None,
                 date_from: datetime.datetime = None, email_mapping=None):
        """

        :param username: The SMTP username to log in with
        :param password:  The SMTP password to log in with
        :param settings:  The SMTP settings. Defaults to GMAIL
        :param email_address: The email address that will be looked for using the TO header. Defaults to username if
            not specified
        :param locations: Optional labels to look under
        :param date_from: How far back to search emails from. Default to 7 days.
        :param email_mapping: dict of Stores - store email to query from
        """
        super(SmtpReader, self).__init__(date_from=date_from, user_email=email_address or username,
                                         email_mapping=email_mapping)
        self.username = username
        self._password = password
        self.mail = imaplib.IMAP4_SSL(*settings.value)
        self.email_locations = locations or {}

    def read_store_emails(self, store: Stores, subject: str = None) -> typing.Iterable[str]:
        location = self.email_locations.get(store)
        if location:
            self.mail.select(location)
        search_query = self._get_search_query(store, subject)
        result, amazon_data = self.mail.uid('search', None, search_query)
        for num in amazon_data[0].split():
            m, v = self.mail.uid("fetch", num, "(RFC822)")
            msg_body = email.message_from_bytes(v[0][1])
            yield msg_body

    def _finish(self):
        self.mail.logout()

    def _login(self):
        self.mail.login(self.username, self._password)
        self.mail.select('inbox')



import base64
import datetime
import re

from bs4 import BeautifulSoup

from email_scrapper.models import Order, Stores, Item


def parse_walmart_email(msg_body):
    msg_body: bytes = base64.b64decode(msg_body.get_payload()[0]._payload)
    email = str(msg_body.decode("utf-8"))
    soup = BeautifulSoup(email, "lxml")
    order_items = soup.find_all("table", {"cellpadding": "5", "cellspacing": "0"})[0].find_all("tr", {"valign": "top"})[
                  1:]
    date = datetime.datetime.strptime(soup.find_all("orderdate")[0].text, "%B %d, %Y")
    order_number = soup.find_all("ordernumber")[0].text
    cart = []
    order_discount = 0.00
    discounts = set(re.findall("-CDN\$ .*", email))
    for discount in discounts:
        try:
            amount = float(discount[6:])
        except:
            amount = 0
        order_discount += amount
    for item in order_items:
        name = item.select("itemname")[0].text
        quantity = float(item.select("quantity")[0].text)
        unit_price = float(item.select("price")[0].text[1:])
        cart.append(Item(name, unit_price, int(quantity), order_number))
    order = Order(order_number, date, Stores.WALMART, cart)
    return order



import base64
import datetime
import email
import logging
import os
import typing
from email.message import Message

from googleapiclient import errors

from email_scrapper.models import Stores
from email_scrapper.readers.base_reader import BaseReader

logger = logging.getLogger(__name__)


class GmailReader(BaseReader):
    SCOPES = ['https://www.googleapis.com/auth/gmail.readonly']

    def __init__(self, service, user_id: str = "me", user_email: str = None, email_mapping: dict = None,
                 date_from: datetime.datetime = None):
        """

        Parameters
        ----------
        service:
        The Gmail API service
        email_mapping: dict
        Mapping of class:Stores: to str representing the email to search from
        """
        super(GmailReader, self).__init__(date_from=date_from, user_email=user_email, email_mapping=email_mapping)
        self.service = service
        self.user_id = user_id

    @classmethod
    def authenticate_with_browser(cls, credentials_json: dict = None, date_from: datetime.datetime = None):
        """
        Login to gmail through the browser.
        Requires a credentials.json file or a credentials_json dict passed

        Returns
        -------
        GmailReader

        """
        try:
            from google_auth_oauthlib.flow import InstalledAppFlow
            from googleapiclient.discovery import build
            import pickle
            creds = None

            if os.path.exists('token.pickle'):
                with open('token.pickle', 'rb') as token:
                    creds = pickle.load(token)
            if not creds or not creds.valid:
                if credentials_json:
                    flow = InstalledAppFlow.from_client_config(credentials_json, GmailReader.SCOPES)
                else:
                    flow = InstalledAppFlow.from_client_secrets_file(
                        'credentials.json', GmailReader.SCOPES)
                creds = flow.run_local_server(port=0)
            # Save the credentials for the next run
            with open('token.pickle', 'wb') as token:
                pickle.dump(creds, token)
            service = build('gmail', 'v1', credentials=creds)
            response = service.users().getProfile(userId="me").execute()
            return cls(service, user_id="me", user_email=response.get("emailAddress"), date_from=date_from)
        except (ImportError, ModuleNotFoundError):
            raise BaseException("Google Auth library not found")

    def _get_search_date_range(self):
        return self.search_date_range.strftime("%Y-%m-%d")

    def _get_email_details(self, message) -> Message:
        response = self.service.users().messages().get(userId=self.user_id, id=message['id'], format="raw").execute()
        msg_str = base64.urlsafe_b64decode(response['raw'].encode('ASCII'))
        mime_msg = email.message_from_bytes(msg_str)
        return mime_msg

    def _get_search_query(self, store: Stores, subject: str = None):
        return f"from:{self._get_store_email(store)} after:{self._get_search_date_range()}"

    def read_store_emails(self, store: Stores, subject: str = None) -> typing.Generator[str, None, None]:
        query = self._get_search_query(store, subject)
        try:
            response = self.service.users().messages().list(userId=self.user_id,
                                                            q=query).execute()
            if 'messages' in response:
                for message in response['messages']:
                    yield self._get_email_details(message)
            while 'nextPageToken' in response:
                page_token = response['nextPageToken']
                response = self.service.users().messages().list(userId=self.user_id, q=query,
                                                                pageToken=page_token).execute()
                for message in response['messages']:
                    yield self._get_email_details(message)
        except errors.HttpError as error:
            print('An error occurred: %s' % error)



import os

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

extra_requires = {
    "gmail": ["google-api-python-client"]
}

setuptools.setup(
    name='email_scrapper',
    version=os.getenv('VERSION', '0.6.0'),
    author="LucasCLuk",
    description="An email parser for store orders.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/LucasCLuk/email-scrapper",
    packages=setuptools.find_packages(),
    install_requires=requirements,
    extra_requires=extra_requires,
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
